
/******************************************************************************
*
*   FILE
*   ----
*   xdrClientHandler.c
*
*   History
*   -------
*   2013-01-29   File created
*
*******************************************************************************
*
*   This file is generated by the 'acplt_builder' command
*
******************************************************************************/


#ifndef OV_COMPILE_LIBRARY_ksxdr
#define OV_COMPILE_LIBRARY_ksxdr
#endif



#include "ksxdr.h"
#include "ksxdr_config.h"
#include "libov/ov_macros.h"
#include "libov/ov_path.h"
#include "libov/ov_memstack.h"
#include "libov/ov_result.h"
#include "ks_logfile.h"
#include "KSDATAPACKET_xdrhandling.h"
#include "ov_ksserver_backend.h"
#include "ksbase_helper.h"
#include "NoneTicketAuthenticator.h"
#include "ksxdr_services.h"


OV_DLLFNCEXPORT void ksxdr_xdrClientHandler_startup(
	OV_INSTPTR_ov_object 	pobj
) {
    /*    
    *   local variables
    */

    /* do what the base class does first */
    ksbase_ClientHandler_startup(pobj);

    /* do what */


    return;
}

OV_DLLFNCEXPORT void ksxdr_xdrClientHandler_shutdown(
	OV_INSTPTR_ov_object 	pobj
) {
    /*    
    *   local variables
    */

    /* do what */

    /* set the object's state to "shut down" */
    ksbase_ClientHandler_shutdown(pobj);

    return;
}

void ksxdr_prepend_length(KS_DATAPACKET* answer, OV_UINT begin)
{
	unsigned int i;
	unsigned int temp;
	if(answer && answer->data && answer->length)
	{
		temp = answer->length - 4;
		for(i=0; i<4; i++)
			answer->data[begin+3-i] = ((OV_BYTE*)&temp)[i];
		answer->data[begin] = 0x80;
	}
	return;
}

OV_RESULT ksxdr_create_global_answer(KS_DATAPACKET* answer, const OV_UINT xid, const OV_UINT msgAccepted, const OV_UINT msgState)
{
	OV_RESULT result;
	OV_UINT dummy;

	/*	xid is always here	*/
	result = KS_DATAPACKET_write_xdr_uint(answer, &xid);
	if(Ov_Fail(result))
		return result;

		/*	with this function we always put together a reply	*/
	dummy = KSXDR_MSGTYPE_REPLY;
	result = KS_DATAPACKET_write_xdr_uint(answer, &dummy);
	if(Ov_Fail(result))
		return result;

	/*	acceptance	*/
	result = KS_DATAPACKET_write_xdr_uint(answer, &msgAccepted);
	if(Ov_Fail(result))
		return result;

	if(msgAccepted != XDR_MSG_ACCEPTED)
	{	/*	rejected add reason	*/
		/*	reason	*/
		result = KS_DATAPACKET_write_xdr_uint(answer, &msgState);
		if(Ov_Fail(result))
			return result;

		if(msgState == XDR_DEN_AUTH_ERROR)
		{/*	rejection because of bad authentication	*/
			dummy = XDR_AUTH_FAILED;
			result = KS_DATAPACKET_write_xdr_uint(answer, &dummy);
			if(Ov_Fail(result))
				return result;

		}
		else
		{/*	rejection because of rpc mismatch --> return highest and lowest supported version (here both 2)	*/
			dummy = 0x02;
			result = KS_DATAPACKET_write_xdr_uint(answer, &dummy);
			if(Ov_Fail(result))
				return result;

			result = KS_DATAPACKET_write_xdr_uint(answer, &dummy);
			if(Ov_Fail(result))
				return result;

		}
		/*	rejection message finished	*/
		return OV_ERR_OK;
	}

	/*	message accepted	*/
	/*	auth verifier	--> no auth*/
	dummy = XDR_AUTH_NONE;
	result = KS_DATAPACKET_write_xdr_uint(answer, &dummy);
	if(Ov_Fail(result))
		return result;

	dummy = 0;
	result = KS_DATAPACKET_write_xdr_uint(answer, &dummy);	/*	no auth data following (length 0)	*/
	if(Ov_Fail(result))
		return result;

	/*	message state	*/
	result = KS_DATAPACKET_write_xdr_uint(answer, &msgState);
	if(Ov_Fail(result))
		return result;

	if(msgState != XDR_MSGST_SUCCESS)
	{	/*	processing failed	*/
		if(msgState == XDR_MSGST_PROG_MISMATCH)
		{/*rejection because of program version mismatch -->return highest and lowest supported ks-version (here both 2)	*/
			dummy = 0x02;
			result = KS_DATAPACKET_write_xdr_uint(answer, &dummy);
			if(Ov_Fail(result))
				return result;
			result = KS_DATAPACKET_write_xdr_uint(answer, &dummy);
			if(Ov_Fail(result))
				return result;
		}
	}
		/*	global part finished	*/
	return OV_ERR_OK;
}

/*
 * This function checks if the input buffer holds a complete request
 */

OV_BOOL bufferHoldsCompleteRequest(KS_DATAPACKET* dataReceived, OV_BYTE* BeginOfMessage, OV_RESULT* result)
{
	OV_BYTE* tempread;
	OV_UINT length;
	OV_UINT header;

	KS_logfile_debug(("checking for complete request: dataREceived:\n\tdata\t%p\n\tlength\t%lu\n\tdata-end\t%p\nread-pt\t%p\n\twrite-pt\t%p\nBeginofMessage\t%p\n\n", dataReceived->data, dataReceived->length, dataReceived->data+dataReceived->length, dataReceived->readPT, dataReceived->writePT, BeginOfMessage));

	tempread = dataReceived->readPT;	/*	store the original read pointer to reset is at the end of the function	*/


	dataReceived->readPT = BeginOfMessage;
	*result = KS_DATAPACKET_read_xdr_uint(dataReceived, &header);	/*	read out header	*/
	if(Ov_Fail(*result))
		return FALSE;

	length = (header & 0x00ffffff);		/*	mask away the most significant byte as this does not contain length information	*/
	if(header & 0x80000000)	/*	first byte of header == 0x80 means this is the last fragment	*/
	{
		KS_logfile_debug(("checking for complete request: last fragment"));
		if((dataReceived->length - (dataReceived->readPT - dataReceived->data)) >= length)		/*	check if there are more than length bytes behind the read pointer	*/
		{
			dataReceived->readPT = tempread;
			KS_logfile_debug(("checking for complete request: request complete"));
			return TRUE;
		}
		else
		{
			dataReceived->readPT = tempread;
			return FALSE;
		}
	}
	else	/*	this is not the last fragment --> iterate over buffer	*/
	{
		if((dataReceived->length - (dataReceived->readPT - dataReceived->data)) > length)		/*	check if there are more than length bytes behind the read pointer	*/
		{
			dataReceived->readPT += length;

			if(bufferHoldsCompleteRequest(dataReceived, BeginOfMessage + length + 4, result))	/*	check the next fragment recursively	*/
			{
				dataReceived->readPT = tempread;
				return TRUE;
			}
			else
			{
				dataReceived->readPT = tempread;
				return FALSE;
			}
		}
		else
		{
			KS_logfile_debug(("checking for complete request: buffer ends before or at end of current fragment --> message not complete"));
			dataReceived->readPT = tempread;
			return FALSE;
		}
	}

}

/*
 * This function check if an xdr message is fragmented. if it is the fragment headers (4 bytes containing the length of the fragment)
 * are removed from the stream and the first bytes of the overall message will contain its length.
 * This means that we can decode messages of up to 2^24 (about 16 mega-) bytes of length
 */

OV_RESULT unfragmentXDRmessage(KS_DATAPACKET* dataReceived, OV_BYTE* BeginOfMessage)
{
	OV_BYTE* tempread;
	OV_BYTE* tempwrite;
	OV_UINT length = 0;
	OV_UINT fragmentLength;
	OV_UINT fragmentHeader;
	OV_RESULT result;
	OV_UINT* writeptr = NULL;
	OV_UINT* readptr = NULL;


	tempread = dataReceived->readPT;	/*	tore pointers to reset them before returning	*/
	tempwrite = dataReceived->writePT;

	dataReceived->readPT = BeginOfMessage;
	dataReceived->writePT = BeginOfMessage;


	result = KS_DATAPACKET_read_xdr_uint(dataReceived, &fragmentHeader);	/*	first fragment	*/
	if(Ov_Fail(result))
		return result;
	dataReceived->writePT += 4;	/*	let writePT point to the same spot as readPT	*/
	fragmentLength = fragmentHeader & 0x00ffffff;	/*	zero most significant byte (does not contain length information)	*/
	length = fragmentLength + 4;	/*	up to now just one fragment and 4 bytes header	*/
	dataReceived->writePT += fragmentLength;	/*	let writePT and readPT point behind the first fragment	*/
	dataReceived->readPT += fragmentLength;
	while(!(fragmentHeader & 0x80000000))	/*	while the current fragment is not the last one	*/
	{
		result = KS_DATAPACKET_read_xdr_uint(dataReceived, &fragmentHeader);	/*	"next" fragment;
																					from here on, readPT is
																					4 bytes per cycle in
																					front of writePT	*/
		if(Ov_Fail(result))
		{
			dataReceived->readPT = tempread;
			dataReceived->writePT = tempwrite;
			return result;
		}
		fragmentLength = fragmentHeader & 0x00ffffff;	/*	zero most significant byte (does not contain length information)	*/
		length += fragmentLength;
		writeptr = (OV_UINT*) dataReceived->writePT;	/*	these pointers are used to need less typecasts	*/
		readptr = (OV_UINT*) dataReceived->readPT;
		while(writeptr < (OV_UINT*) (dataReceived->data + length)
				&& (length < dataReceived->length)
				&& readptr < (OV_UINT*) (dataReceived->data + dataReceived->length))
		{
			*writeptr = *readptr;	/*	xdrs are alingned in 4 byte blocks so we can copy 4 bytes per cycle (speedup)	*/
			writeptr++;
			readptr++;
		}
		dataReceived->readPT = (OV_BYTE*) readptr;
		dataReceived->writePT = (OV_BYTE*) writeptr;
	}

	dataReceived->readPT = tempread;
	dataReceived->writePT = tempwrite;
	dataReceived->length = length; /*	length of all fragments plus 4 bytes header of the first one	*/
	return OV_ERR_OK;
}

/**
 * This function handles requests received over a channel. It reads data as an xdr-stream and triggers the appropriate OV-functions
 */

OV_DLLFNCEXPORT OV_RESULT ksxdr_xdrClientHandler_HandleRequest(
	OV_INSTPTR_ksbase_ClientHandler this,
	OV_INSTPTR_ksbase_Channel pChannel,
	KS_DATAPACKET* dataReceived,
	KS_DATAPACKET* answer
) {
    /*    
    *   local variables
    */
	OV_RESULT result;
	OV_UINT msgAccepted = XDR_MSG_ACCEPTED;	/* Acceptance state for message*/
	OV_UINT	msgState = XDR_MSGST_SUCCESS;	/* Error code for rpc layer*/
	OV_UINT ksErrCode = KS_ERR_OK;			/* Error code on ks layer*/
	OV_UINT header;
	OV_UINT xid;
	OV_UINT messageType;
	OV_UINT rpcVersion;
	OV_UINT progId;
	OV_UINT ProgVersion;
	OV_UINT procedure;
	OV_UINT authflavour = 0;					/*also used as ticketindicator*/
	OV_STRING authdata = NULL;
	OV_UINT decodeLength = 0;
	OV_TICKET* pticket = NULL;
	OV_INSTPTR_ksbase_TicketAuthenticator pTicketAuth = NULL;
	OV_INSTPTR_ov_domain pDomAuthenticators	= NULL;
	KS_DATAPACKET serviceAnswer = {0, NULL, NULL, NULL};
	OV_UINT beginAnswer = 0;	/*	indicates the beginning of the answer in the packet, set before create_global_answer, used by prepend_length	*/
	OV_BYTE* BeginOfMessage = NULL;	/*	stores the pointer to the very beginning of the message.
										this is needed if the message is valid but incomplete and has to be processed in the next cycle	*/
	OV_UINT dummy;
	OV_INSTPTR_ksxdr_xdrClientHandler thisxdrCL	=	Ov_StaticPtrCast(ksxdr_xdrClientHandler, this);
	OV_INSTPTR_ksxdr_xdrClientHandlerExtension	pExtension	=	NULL;
	OV_VTBLPTR_ksxdr_xdrClientHandlerExtension	pVtblClientHandlerExtension	=	NULL;
	OV_INSTPTR_ov_class							pExtensionClass	=	NULL;
	OV_UINT	iterator;
	OV_BOOL found;
	OV_BOOL	extensionTakesOver	=	FALSE;

	/*********************************************************************************************************************************************************************
	 * decode header
	 ********************************************************************************************************************************************************************/

	if(pChannel->v_usesStreamProtocol == TRUE)
	{
		BeginOfMessage = dataReceived->readPT;
		if(Ov_Fail(KS_DATAPACKET_read_xdr_uint(dataReceived, &header)))				//not part of rpc call
		{
			KS_logfile_error(("%s: HandleRequest: cold not decode stream protocol header", this->v_identifier));
			return OV_ERR_GENERIC;
		}
	}

	if(Ov_Fail(KS_DATAPACKET_read_xdr_uint(dataReceived, &xid))					//offset 0 in rpc call
			|| Ov_Fail(KS_DATAPACKET_read_xdr_uint(dataReceived, &messageType))	//offset 4
			|| Ov_Fail(KS_DATAPACKET_read_xdr_uint(dataReceived, &rpcVersion))	//offset 8
			|| Ov_Fail(KS_DATAPACKET_read_xdr_uint(dataReceived, &progId))		//offset 12
			|| Ov_Fail(KS_DATAPACKET_read_xdr_uint(dataReceived, &ProgVersion))	//offset 16
			|| Ov_Fail(KS_DATAPACKET_read_xdr_uint(dataReceived, &procedure)))	//offset 20
	{
		KS_logfile_error(("%s: HandleRequest: cold not decode rpc-header", this->v_identifier));
		KS_logfile_debug(("datapacket: \n\tlength:\t\t%lu\n\tdata:\t\t%p\n\treadPT:\t\t%p\n\twritePt:\t%p\n", dataReceived->length, dataReceived->data, dataReceived->readPT, dataReceived->writePT));
		return OV_ERR_GENERIC;
	}

	KS_logfile_debug(("%s: Header values:\n\theader\t%0#10x\n\txid\t\t%0#10x\n\tmessageType\t%0#10x\n\trpcVersion\t%0#10x\n\tprogId\t\t%0#10x\n\tProgVersion\t%0#10x\n\tprocedure\t%0#10x\n\n",
				this->v_identifier, header, xid, messageType, rpcVersion, progId, ProgVersion, procedure));

	/****************************************************************************************************************************************************************************
	 * Check if data is valid
	 ***************************************************************************************************************************************************************************/
	if((rpcVersion > KSXDR_RPCVERSION))
	{
		msgAccepted = XDR_MSG_DENIED;
		msgState = XDR_DEN_RPC_MISMATCH;
		KS_logfile_error(("%s: HandleRequest: rpcVersion is not valid: rpcVersion\t%0#8x\tshould be %0#8x", this->v_identifier, rpcVersion, KSXDR_RPCVERSION));
	}
	else if(progId != KSXDR_PROGRAMNUMBER)
	{
		msgState = XDR_MSGST_PROG_UNAVAIL;
		KS_logfile_error(("%s: HandleRequest: ProgramID is not valid: ProgramID\t%0#8x\tshould be %0#8x", this->v_identifier, progId, KSXDR_PROGRAMNUMBER));
	}
	else if(ProgVersion > KSXDR_PROGRAMVERSION)
	{
		msgState = XDR_MSGST_PROG_MISMATCH;
		KS_logfile_error(("%s: HandleRequest: program version mismatch: program version\t%0#8x\tmust be less than be %0#8x", this->v_identifier, ProgVersion, KSXDR_PROGRAMVERSION));
	}
	else if(messageType != KSXDR_MSGTYPE_CALL)
	{
		KS_logfile_error(("%s: HandleRequest: wrong message type received: \t%0#8x\tshould be %0#8x", this->v_identifier, messageType, KSXDR_MSGTYPE_CALL));
		return OV_ERR_GENERIC;
	}

	/***************************************************************************************************************************************************************************
	 * Handle errors up to here
	 **************************************************************************************************************************************************************************/
	if((msgAccepted != XDR_MSG_ACCEPTED)
			|| (msgState != XDR_MSGST_SUCCESS))
	{
		KS_logfile_error(("%s: HandleRequest: Message denied or decoding not successfull. Sending Answer.", this->v_identifier));
		if(pChannel->v_usesStreamProtocol == TRUE)
		{
			/*	save index of first byte of answer in packet in beginAnswer	*/
			beginAnswer = ((answer->writePT - answer->data));
			/*	reserver space for length (first 4 bytes in xdr 0x80xxxxxx) length = xxxxxx	*/
			dummy = 0x80;
			result = KS_DATAPACKET_write_xdr_uint(answer, &dummy);
			if(Ov_Fail(result))
					return result;

		}
		ksxdr_create_global_answer(answer, xid, msgAccepted, msgState);
		if(pChannel->v_usesStreamProtocol == TRUE)
			ksxdr_prepend_length(answer, beginAnswer);
		return OV_ERR_OK;		/*	this function worked properly, the error is encoded in the reply message	*/
	}

	/***************************************************************************************************************************************************************************
	 * checking if request is in the buffer completely, if not, return ERR_OK and wait until it is complete. While waiting set the receive timeout value to a few seconds
	 * 	(we received a fragmented package so there will come more data. if no data arrives within a second, we can assume the package is broken)
	 ***************************************************************************************************************************************************************************/
	if(pChannel->v_usesStreamProtocol == TRUE)
	{
		if(!bufferHoldsCompleteRequest(dataReceived, BeginOfMessage, &result))
		{
			if(Ov_Fail(result))
			{
				KS_logfile_error(("%s: HandleRequest: error checking if message is complete: %s", this->v_identifier, ov_result_getresulttext(result)));
				KS_logfile_error(("datapackets content: \nlength:\t%lu\ndata-begin:\t%p\ndata-end:\t%p\nread-pt:\t%p\nwrite-pt:\t%p\n\n", dataReceived->length, dataReceived->data, dataReceived->data+dataReceived->length, dataReceived->readPT, dataReceived->writePT));
				return result;
			}
			KS_logfile_debug(("%s: HandleRequest: Buffer does NOT hold the complete request. waiting some time...", this->v_identifier));
			pChannel->v_ConnectionTimeOut = thisxdrCL->v_timeoutIncomplete;
			dataReceived->readPT = BeginOfMessage; 	/*	reset the read pointer to the place where the message began (to be read again next time)	*/
			return OV_ERR_OK;		/*	get called again to process the request next time (if it is complete then).
											Yes, this could block the ClientHandler for a longer time.	*/
		}
		else
		{
			pChannel->v_ConnectionTimeOut = thisxdrCL->v_connectionTimeout;
		}

		/*
		 * unfragment large xdrs
		 */
		if(!(header & (0x80000000)))	/*	a 0x80 in the first byte means this is the last (or only in this case) fragment. 	*/
		{
			KS_logfile_debug(("%s: defragmenting rpc message", this->v_identifier));
			result = unfragmentXDRmessage(dataReceived, BeginOfMessage);
			if(Ov_Fail(result))
				return result;
			KS_logfile_debug(("%s: message defragmented", this->v_identifier));

		}
	}
	/***************************************************************************************************************************************************************************
	 * first part of message decoded. Now the xdr auth-parts follow: we do not use them.
	 * there auth_credentials and auth_verification. Both contain up to 400 bytes opaque data (variable length)
	 **************************************************************************************************************************************************************************/
			/*credentials*/
	if(Ov_Fail(KS_DATAPACKET_read_xdr_uint(dataReceived, &authflavour)))
	{
		KS_logfile_error(("%s: HandleRequest: reading auth_cred flavor failed", this->v_identifier));
		msgAccepted = XDR_MSG_DENIED;
		msgState = XDR_DEN_AUTH_ERROR;
	}
	else if(authflavour != XDR_AUTH_NONE)
	{
		KS_logfile_error(("%s: HandleRequest: auth_cred flavor NOT NONE", this->v_identifier));
		msgAccepted = XDR_MSG_DENIED;
		msgState = XDR_DEN_AUTH_ERROR;
	}
	if(Ov_Fail(KS_DATAPACKET_read_xdr_opaque(dataReceived, &authdata, &decodeLength, KS_SIMPLETICKETID_MAXLEN)))
	{
		KS_logfile_error(("%s: HandleRequest: reading auth_cred data failed", this->v_identifier));
		msgAccepted = XDR_MSG_DENIED;
		msgState = XDR_DEN_AUTH_ERROR;
	}
			/*verifier*/
	if(Ov_Fail(KS_DATAPACKET_read_xdr_uint(dataReceived, &authflavour)))
	{
		KS_logfile_error(("%s: HandleRequest: reading auth_ver flavor failed", this->v_identifier));
		msgAccepted = XDR_MSG_DENIED;
		msgState = XDR_DEN_AUTH_ERROR;
	}
	else if(authflavour != XDR_AUTH_NONE)
	{
		KS_logfile_error(("%s: HandleRequest: auth_ver flavor NOT NONE", this->v_identifier));
		msgAccepted = XDR_MSG_DENIED;
		msgState = XDR_DEN_AUTH_ERROR;
	}
	if(Ov_Fail(KS_DATAPACKET_read_xdr_opaque(dataReceived, &authdata, &decodeLength, KS_SIMPLETICKETID_MAXLEN)))
	{
		KS_logfile_error(("%s: HandleRequest: reading auth_ver data failed", this->v_identifier));
		msgAccepted = XDR_MSG_DENIED;
		msgState = XDR_DEN_AUTH_ERROR;
	}

	/************************************************************************************************************************************************************************
	 * Handle errors up to here
	 ************************************************************************************************************************************************************************/
	if(msgAccepted != XDR_MSG_ACCEPTED)
	{
		KS_logfile_error(("%s: HandleRequest: Message denied. Sending Answer.", this->v_identifier));
		if(pChannel->v_usesStreamProtocol == TRUE)
		{
			/*	save index of first byte of answer in packet in beginAnswer	*/
			beginAnswer = ((answer->writePT - answer->data));
			/*	reserve space for length (first 4 bytes in xdr 0x80xxxxxx) length = xxxxxx	*/
			dummy = 0x80;
			result = KS_DATAPACKET_write_xdr_uint(answer, &dummy);
			if(Ov_Fail(result))
				return result;

		}
		ksxdr_create_global_answer(answer, xid, msgAccepted, msgState);
		if(pChannel->v_usesStreamProtocol == TRUE)
			ksxdr_prepend_length(answer, beginAnswer);
		return OV_ERR_OK;		/*	this function worked properly, the error is encoded in the reply message	*/
	}
	/***************************************************************************************************************************************************************************
	 * the rpc-part of the message is valid --> we received a service-request --> increment receivedcalls
	 **************************************************************************************************************************************************************************/
		this->v_receivedCalls++;

	/***************************************************************************************************************************************************************************
	 * handle ticket indicator; offset now should be 36 (since the auth blocks should be 8 bytes each (opaque data length 0))
	 **************************************************************************************************************************************************************************/
		/* read ticket indicator*/
	if(Ov_Fail(KS_DATAPACKET_read_xdr_uint(dataReceived, &authflavour)))
	{
		KS_logfile_error(("%s: HandleRequest: reading ticketindicator failed", this->v_identifier));
		msgState = XDR_MSGST_SUCCESS;
		ksErrCode = KS_ERR_TARGETGENERIC;
	}

	/**********************************************************************************************************************************************************************
	 * The following code processes tickets
	 *********************************************************************************************************************************************************************/

	if(authflavour == OV_TT_NONE)
	{
		pticket = ksbase_NoneAuth->v_ticket.vtbl->createticket(NULL, OV_TT_NONE);
	}
	else
	{
		pDomAuthenticators = Ov_StaticPtrCast(ov_domain, ov_path_getobjectpointer("/communication/ksxdr/TicketAuthenticators", 2));
		if(!pDomAuthenticators)
		{
			KS_logfile_error(("%s: HandleRequest: Fatal: No TicketAuthenticators installed", this->v_identifier));
			msgState = XDR_MSGST_SYSTEM_ERR;
			ksErrCode = KS_ERR_TARGETGENERIC;
		}
		else
		{
			Ov_ForEachChildEx(ov_containment, pDomAuthenticators, pTicketAuth, ksbase_TicketAuthenticator)
			{
					/*	check if authenticator fits for ticket type	*/
				if((OV_UINT) pTicketAuth->v_TicketType == authflavour)
				{
					pticket = pTicketAuth->v_ticket.vtbl->createticket((void*) dataReceived, authflavour);
					if(!pticket)
					{
						KS_logfile_error(("%s: HandleRequest: Decoding ticket failed", this->v_identifier));
						msgState = XDR_MSGST_SYSTEM_ERR;
						ksErrCode = KS_ERR_TARGETGENERIC;
					}
				}
				else
				{
					KS_logfile_error(("%s: HandleRequest: No TicketAuthenticator for this TicketType", this->v_identifier));
					msgState = XDR_MSGST_SUCCESS;
					ksErrCode = KS_ERR_UNKNOWNAUTH;
				}
			}
		}
	}

	/***************************************************************************************************************************************************************************
	 * Handle errors up to here
	 **************************************************************************************************************************************************************************/
	if(msgState != XDR_MSGST_SUCCESS)
	{
		KS_logfile_error(("%s: HandleRequest: Error while authenticating. Sending Answer.", this->v_identifier));
		if(pChannel->v_usesStreamProtocol == TRUE)
		{
			/*	save index of first byte of answer in packet in beginAnswer	*/
			beginAnswer = ((answer->writePT - answer->data));
			/*	reserver space for length (first 4 bytes in xdr 0x80xxxxxx) length = xxxxxx	*/
			dummy = 0x80;
			result = KS_DATAPACKET_write_xdr_uint(answer, &dummy);
			if(Ov_Fail(result))
				return result;

		}
		ksxdr_create_global_answer(answer, xid, msgAccepted, msgState);
		if(pChannel->v_usesStreamProtocol == TRUE)
			ksxdr_prepend_length(answer, beginAnswer);
		return OV_ERR_OK;		/*	this function worked properly, the error is encoded in the reply message	*/
	}


	/************************************************************************************************************************************************************************
	 * Dispatch ks-procedures / services
	 ***********************************************************************************************************************************************************************/

	switch(procedure)
	{
	case KS_REGISTER:
		KS_logfile_debug(("%s: HandleRequest: processing REGISTER (%0#8x)", this->v_identifier, procedure));

		if(!pChannel->v_isLocal)
		{
			ksErrCode = KS_ERR_NOREMOTE;
			break;
		}
		result = ksxdr_register(ProgVersion, pticket, dataReceived, &serviceAnswer, &msgState, &ksErrCode);
		if(Ov_Fail(result))
		{
			ov_memstack_lock();
			KS_logfile_error(("%s: HandleRequest: processing service %0#8x failed with error %s", this->v_identifier, procedure, ov_result_getresulttext(result)));
			ov_memstack_unlock();
			ksbase_free_KSDATAPACKET(&serviceAnswer);
			ksbase_free_KSDATAPACKET(answer);
			return result;
		}
		break;
		/***********************************************************************************************************************************************************************/

	case KS_UNREGISTER:
		KS_logfile_debug(("%s: HandleRequest: processing UNREGISTER (%0#8x)", this->v_identifier, procedure));

		if(!pChannel->v_isLocal)
		{
			ksErrCode = KS_ERR_NOREMOTE;
			break;
		}
		result = ksxdr_unregister(ProgVersion, pticket, dataReceived, &serviceAnswer, &msgState, &ksErrCode);
		if(Ov_Fail(result))
		{
			KS_logfile_error(("%s: HandleRequest: processing service %0#8x failed with error %s", this->v_identifier, procedure, ov_result_getresulttext(result)));
			ksbase_free_KSDATAPACKET(&serviceAnswer);
			ksbase_free_KSDATAPACKET(answer);
			return result;
		}
		break;
		/***********************************************************************************************************************************************************************/

	case KS_GETSERVER:
		KS_logfile_debug(("%s: HandleRequest: processing GETSERVER (%0#8x)", this->v_identifier, procedure));
		result = ksxdr_getserver(ProgVersion, pticket, dataReceived, &serviceAnswer, &msgState, &ksErrCode);
		if(Ov_Fail(result))
		{
			KS_logfile_error(("%s: HandleRequest: processing service %0#8x failed with error %s", this->v_identifier, procedure, ov_result_getresulttext(result)));
			ksbase_free_KSDATAPACKET(&serviceAnswer);
			ksbase_free_KSDATAPACKET(answer);
			return result;
		}
		break;
		/***********************************************************************************************************************************************************************/

	case KS_CREATEOBJECT:
		KS_logfile_debug(("%s: HandleRequest: processing CREATEOBJECT (%0#8x)", this->v_identifier, procedure));
		result = ksxdr_createObject(ProgVersion, pticket, dataReceived, &serviceAnswer, &msgState, &ksErrCode);
		if(Ov_Fail(result))
		{
			KS_logfile_error(("%s: HandleRequest: processing service %0#8x failed with error %s", this->v_identifier, procedure, ov_result_getresulttext(result)));
			ksbase_free_KSDATAPACKET(&serviceAnswer);
			ksbase_free_KSDATAPACKET(answer);
			return result;
		}
		break;
		/***********************************************************************************************************************************************************************/

	case KS_DELETEOBJECT:
		KS_logfile_debug(("%s: HandleRequest: processing DELETEOBJECT (%0#8x)", this->v_identifier, procedure));
		result = ksxdr_deleteObject(ProgVersion, pticket, dataReceived, &serviceAnswer, &msgState, &ksErrCode);
		if(Ov_Fail(result))
		{
			KS_logfile_error(("%s: HandleRequest: processing service %0#8x failed with error %s", this->v_identifier, procedure, ov_result_getresulttext(result)));
			ksbase_free_KSDATAPACKET(&serviceAnswer);
			ksbase_free_KSDATAPACKET(answer);
			return result;
		}
		break;
		/***********************************************************************************************************************************************************************/

	case KS_GETEP:
		KS_logfile_debug(("%s: HandleRequest: processing GETEP (%0#8x)", this->v_identifier, procedure));
		result = ksxdr_getEP(ProgVersion, pticket, dataReceived, &serviceAnswer, &msgState, &ksErrCode);
		if(Ov_Fail(result))
		{
			KS_logfile_error(("%s: HandleRequest: processing service %0#8x failed with error %s", this->v_identifier, procedure, ov_result_getresulttext(result)));
			ksbase_free_KSDATAPACKET(&serviceAnswer);
			ksbase_free_KSDATAPACKET(answer);
			return result;
		}
		break;
		/***********************************************************************************************************************************************************************/

	case KS_GETPP:
		KS_logfile_debug(("%s: HandleRequest: processing GETPP (%0#8x)", this->v_identifier, procedure));
		result = ksxdr_getPP(ProgVersion, pticket, dataReceived, &serviceAnswer, &msgState, &ksErrCode);
		if(Ov_Fail(result))
		{
			KS_logfile_error(("%s: HandleRequest: processing service %0#8x failed with error %s", this->v_identifier, procedure, ov_result_getresulttext(result)));
			ksbase_free_KSDATAPACKET(&serviceAnswer);
			ksbase_free_KSDATAPACKET(answer);
			return result;
		}
		break;
		/***********************************************************************************************************************************************************************/

	case KS_GETCANONICALPATH:
		KS_logfile_debug(("%s: HandleRequest: processing GETCANONICALPATH (%0#8x)", this->v_identifier, procedure));

		result = ksxdr_getCanonicalPath(ProgVersion, pticket, dataReceived, &serviceAnswer, &msgState, &ksErrCode);
		if(Ov_Fail(result))
		{
			KS_logfile_error(("%s: HandleRequest: processing service %0#8x failed with error %s", this->v_identifier, procedure, ov_result_getresulttext(result)));
			ksbase_free_KSDATAPACKET(&serviceAnswer);
			ksbase_free_KSDATAPACKET(answer);
			return result;
		}
		break;
		/***********************************************************************************************************************************************************************/

	case KS_GETVAR:
		KS_logfile_debug(("%s: HandleRequest: processing GETVAR (%0#8x)", this->v_identifier, procedure));
		result = ksxdr_getVar(ProgVersion, pticket, dataReceived, &serviceAnswer, &msgState, &ksErrCode);
		if(Ov_Fail(result))
		{
			KS_logfile_error(("%s: HandleRequest: processing service %0#8x failed with error %s", this->v_identifier, procedure, ov_result_getresulttext(result)));
			ksbase_free_KSDATAPACKET(&serviceAnswer);
			ksbase_free_KSDATAPACKET(answer);
			return result;
		}
		break;
		/***********************************************************************************************************************************************************************/

	case KS_LINK:
		KS_logfile_debug(("%s: HandleRequest: processing LINK (%0#8x)", this->v_identifier, procedure));

		result = ksxdr_link(ProgVersion, pticket, dataReceived, &serviceAnswer, &msgState, &ksErrCode);
		if(Ov_Fail(result))
		{
			KS_logfile_error(("%s: HandleRequest: processing service %0#8x failed with error %s", this->v_identifier, procedure, ov_result_getresulttext(result)));
			ksbase_free_KSDATAPACKET(&serviceAnswer);
			ksbase_free_KSDATAPACKET(answer);
			return result;
		}
		break;
		/***********************************************************************************************************************************************************************/

	case KS_RENAMEOBJECT:
		KS_logfile_debug(("%s: HandleRequest: processing RENAMEOBJECT (%0#8x)", this->v_identifier, procedure));

		result = ksxdr_renameObject(ProgVersion, pticket, dataReceived, &serviceAnswer, &msgState, &ksErrCode);
		if(Ov_Fail(result))
		{
			KS_logfile_error(("%s: HandleRequest: processing service %0#8x failed with error %s", this->v_identifier, procedure, ov_result_getresulttext(result)));
			ksbase_free_KSDATAPACKET(&serviceAnswer);
			ksbase_free_KSDATAPACKET(answer);
			return result;
		}
		break;
		/***********************************************************************************************************************************************************************/

	case KS_SETVAR:
		KS_logfile_debug(("%s: HandleRequest: processing SETVAR (%0#8x)", this->v_identifier, procedure));

		result = ksxdr_setVar(ProgVersion, pticket, dataReceived, &serviceAnswer, &msgState, &ksErrCode);
		if(Ov_Fail(result))
		{
			KS_logfile_error(("%s: HandleRequest: processing service %0#8x failed with error %s", this->v_identifier, procedure, ov_result_getresulttext(result)));
			ksbase_free_KSDATAPACKET(&serviceAnswer);
			ksbase_free_KSDATAPACKET(answer);
			return result;
		}
		break;
		/***********************************************************************************************************************************************************************/

	case KS_UNLINK:
		KS_logfile_debug(("%s: HandleRequest: processing UNLINK (%0#8x)", this->v_identifier, procedure));

		result = ksxdr_unlink(ProgVersion, pticket, dataReceived, &serviceAnswer, &msgState, &ksErrCode);
		if(Ov_Fail(result))
		{
			KS_logfile_error(("%s: HandleRequest: processing service %0#8x failed with error %s", this->v_identifier, procedure, ov_result_getresulttext(result)));
			ksbase_free_KSDATAPACKET(&serviceAnswer);
			ksbase_free_KSDATAPACKET(answer);
			return result;
		}
		break;
		/***********************************************************************************************************************************************************************/
	case KS_EXGDATA:
	case KS_GETHIST:
	default:
		/*	iterate over all xdrClientHandlerExtensions	*/
		found = FALSE;
		Ov_ForEachChildEx(ov_inheritance, pclass_ksxdr_xdrClientHandlerExtension, pExtensionClass, ov_class){
			Ov_ForEachChildEx(ov_instantiation, pExtensionClass, pExtension, ksxdr_xdrClientHandlerExtension){
				for(iterator = 0; iterator < pExtension->v_procedureNumbers.veclen; iterator++){
					if(pExtension->v_procedureNumbers.value[iterator] == procedure){
						found = TRUE;
						break;
					}
				}
				if(found == TRUE){
					break;
				}
			}
			if(found == TRUE){
				break;
			}
		}
		if(found == TRUE){
			KS_logfile_debug(("%s: HandleRequest: processing extension (%0#8x) by protocolExtension %s", this->v_identifier, procedure, pExtension->v_identifier));
			Ov_GetVTablePtr(ksxdr_xdrClientHandlerExtension, pVtblClientHandlerExtension, pExtension);
			if(!pVtblClientHandlerExtension){
				KS_logfile_error(("%s: HandleRequest: vtable pointer of %s not found. aborting. %s", this->v_identifier, pExtension->v_identifier));
				ksbase_free_KSDATAPACKET(&serviceAnswer);
				ksbase_free_KSDATAPACKET(answer);
				return result;
			}
			result = pVtblClientHandlerExtension->m_HandleExtendedRequest(pExtension, thisxdrCL, pChannel, ProgVersion, xid, msgAccepted, msgState, authflavour,
					pticket, procedure, dataReceived, &beginAnswer, answer, &ksErrCode, &serviceAnswer, &extensionTakesOver);
			if(Ov_Fail(result))
			{
				KS_logfile_error(("%s: HandleRequest: processing service %0#8x by %s failed with error %s", this->v_identifier, procedure, pExtension->v_identifier, ov_result_getresulttext(result)));
				ksbase_free_KSDATAPACKET(&serviceAnswer);
				ksbase_free_KSDATAPACKET(answer);
				return result;
			}
			if(extensionTakesOver == TRUE){
				return OV_ERR_OK;
			} else {
				break;
			}
		} else {
			KS_logfile_error(("%s: HandleRequest: procedure %0#8x not implemented", this->v_identifier, procedure));
			msgAccepted = XDR_MSG_ACCEPTED;
			msgState = XDR_MSGST_SUCCESS;
			ksErrCode = KS_ERR_NOTIMPLEMENTED;
			ksbase_free_KSDATAPACKET(dataReceived);
		}
		break;
	}
	/***********************************************************************************************************************************************************************
	 * Create Answer
	 ***********************************************************************************************************************************************************************/
	result = ksxdr_xdrClientHandler_encodeReplyHeader(pChannel, xid, msgAccepted, msgState, authflavour, pticket, &beginAnswer, answer);
	if(Ov_Fail(result)){
		ksbase_free_KSDATAPACKET(&serviceAnswer);
		return result;
	}
	result =  ksxdr_xdrClientHandler_appendKsReplyBody(pChannel, ksErrCode, &serviceAnswer, beginAnswer, answer);
	if(Ov_Fail(result)){
		return result;
	}

	/*	if the whole buffer was decoded free it	*/
	if((dataReceived->readPT - dataReceived->data) >= (OV_INT) dataReceived->length){
		KS_logfile_debug(("%s: HandleRequest: dataReceived decoded completely, freeing it", this->v_identifier));
		ksbase_free_KSDATAPACKET(dataReceived);
	}
    return OV_ERR_OK;
}

/**
 * encodes the ksxdr reply header
 * in case of error the answer datapacket is freed
 */
OV_DLLFNCEXPORT OV_RESULT ksxdr_xdrClientHandler_encodeReplyHeader(
		OV_INSTPTR_ksbase_Channel pChannel,
		const OV_UINT xid,
		const OV_UINT msgAccepted,
		const OV_UINT msgState,
		const OV_UINT authflavour,
		OV_TICKET* pticket,
		OV_UINT* beginAnswer,
		KS_DATAPACKET* answer
){
	OV_RESULT	result;
	OV_UINT		dummy;

	//DEBUG
	// KS_logfile_debug(("%s HandleData line %u: \n\tanswer.length:\t%u\n\tanswer.readPT\t%p\n\tanswer.data\t%p", this->v_identifier, __LINE__, answer->length, answer->readPT,answer->data));
	//DEBUG_END
	/*	create global part of answer, append service specific part (answerMessage) and send it	*/
	if(pChannel->v_usesStreamProtocol == TRUE)
	{
		/*	save index of first byte of answer in packet in beginAnswer	*/
		*beginAnswer = ((answer->writePT - answer->data));
		/*	reserver space for length (first 4 bytes in xdr 0x80xxxxxx) length = xxxxxx	*/
		dummy = 0x80;
		result = KS_DATAPACKET_write_xdr_uint(answer, &dummy);
		if(Ov_Fail(result))
			return result;

	}
	ksxdr_create_global_answer(answer, xid, msgAccepted, msgState);
	//DEBUG
	// KS_logfile_debug(("%s HandleData line %u: \n\tanswer.length:\t%u\n\tanswer.readPT\t%p\n\tanswer.data\t%p", this->v_identifier, __LINE__, answer->length, answer->readPT,answer->data));
	//DEBUG_END

	/*	set auth part in reply. set tickettype and use ticket->encodereply for specific data	*/
	/*	set ticketindicator	*/
	if(Ov_Fail(KS_DATAPACKET_write_xdr_uint(answer, &authflavour)))	{
		ksbase_free_KSDATAPACKET(answer);
		return OV_ERR_HEAPOUTOFMEMORY;
	}

	/*	set ticketreply	*/
	if(!(pticket->vtbl->encodereply(answer, pticket))){	/*	this will crash if someone tries to delete the ticket-authenticator he is using at the moment	*/
		KS_logfile_error(("encodeRequestHeader: ticket encodereply failed"));
		return OV_ERR_GENERIC;
	}

	/*	delete Ticket	*/
	pticket->vtbl->deleteticket(pticket);
	return OV_ERR_OK;
}

/**
 * appends the body of a ks reply to the datapacket
 * in case of error serviceAnswer and answer are freed
 */
OV_DLLFNCEXPORT OV_RESULT ksxdr_xdrClientHandler_appendKsReplyBody(
		OV_INSTPTR_ksbase_Channel pChannel,
		const OV_UINT ksErrCode,
		KS_DATAPACKET* serviceAnswer,
		OV_UINT beginAnswer,
		KS_DATAPACKET* answer
){
	/*	set ksErrCode	*/
	if(Ov_Fail(KS_DATAPACKET_write_xdr_uint(answer, &ksErrCode)))
	{
		ksbase_free_KSDATAPACKET(serviceAnswer);
		ksbase_free_KSDATAPACKET(answer);
		return OV_ERR_HEAPOUTOFMEMORY;
	}
	/*	append service specific data (if service succeeded)	*/
	if(ksErrCode == KS_ERR_OK)
		if(serviceAnswer->length)
		{
			if(Ov_Fail(ksbase_KSDATAPACKET_append(answer, serviceAnswer->data, serviceAnswer->length)))
			{
				ksbase_free_KSDATAPACKET(serviceAnswer);
				ksbase_free_KSDATAPACKET(answer);
				return OV_ERR_HEAPOUTOFMEMORY;
			}
			ksbase_free_KSDATAPACKET(serviceAnswer);
		}
	if(pChannel->v_usesStreamProtocol == TRUE){
		ksxdr_prepend_length(answer, beginAnswer);
	}
	return OV_ERR_OK;
}
